<?php
/**
 * Plugin Name:     Easy Digital download yTask Payment Gateway
 * Plugin URI:      https://ytask.org/
 * Description:     yTask payment plugin for Easy Digital Downloads.
 * Version:         1.2
 * Author:          Justus Ochieng & opskill.com
 * Author URI:      https://ytask.org/
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'EDD_yTaskPayments' ) ) {

    class EDD_yTaskPayments {

        private static $instance;

        /**
         * Get active instance
         *
         * @since       1.0.0
         * @access      public
         * @static
         * @return      object self::$instance
         */

        public static function get_instance() {
            if( !self::$instance )
                self::$instance = new EDD_yTaskPayments();

            return self::$instance;
        }


        /**
         * Class constructor
         *
         * @since       1.0.0
         * @access      public
         * @return      void
         */
        public function __construct() {
            // Plugin dir
            define( 'EDD_yTaskPAYMENTS_DIR', plugin_dir_path( __FILE__ ) );

            // Plugin URL
            define( 'EDD_yTaskPAYMENTS_URL', plugin_dir_url( __FILE__ ) );

            $this->init();
        }


        /**
         * Run action and filter hooks
         *
         * @since       1.0.0
         * @access      private
         * @return      void
         */
        private function init() {
            // Make sure EDD is active
            if( !class_exists( 'Easy_Digital_Downloads' ) ) return;

            global $edd_options;

            // Internationalization
            add_action( 'init', array( $this, 'textdomain' ) );

            // Register settings
            add_filter( 'edd_settings_gateways', array( $this, 'settings' ), 1 );

            // Add the gateway
            add_filter( 'edd_payment_gateways', array( $this, 'register_gateway' ) );

            // Remove CC form
            add_action( 'edd_yTaskpayments_cc_form', '__return_false' );

            // Process payment
            add_action( 'edd_gateway_yTaskpayments', array( $this, 'process_payment' ) );
            add_action( 'init', array( $this, 'edd_listen_for_yTaskpayments_ipn' ) );
            add_action( 'edd_verify_yTaskpayments_ipn', array( $this, 'edd_process_yTaskpayments_ipn' ) );
            add_action( 'edd_taskipn', array( $this, 'edd_taskipn' ) );

            // Display errors
            add_action( 'edd_after_cc_fields', array( $this, 'errors_div' ), 999 );
        }

    

        /**
         * Internationalization
         *
         * @since       1.0.0
         * @access      public
         * @static
         * @return      void
         */
        public static function textdomain() {
            // Set filter for language directory
            $lang_dir = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
            $lang_dir = apply_filters( 'edd_yTaskpayments_lang_dir', $lang_dir );

            // Load translations
            load_plugin_textdomain( 'edd-yTaskpayments', false, $lang_dir );
        }


        /**
         * Add settings
         *
         * @since       1.0.0
         * @access      public
         * @param       array $settings The existing plugin settings
         * @return      array
         */
        public function settings( $settings ) {
            $yTaskpayments_settings = array(
                array(
                    'id'    => 'edd_yTaskpayments_settings',
                    'name'  => '<strong>' . __( 'yTaskPayments Settings', 'edd-yTaskpayments' ) . '</strong>',
                    'desc'  => __( 'Configure your yTaskPayments settings', 'edd-yTaskpayments' ),
                    'type'  => 'header'
                ),
                array(
                    'id'    => 'edd_yTaskpayments_merchant',
                    'name'  => __( 'yTask API Key', 'edd-yTaskpayments' ),
                    'desc'  => __( 'Enter your yTask APIKEY', 'edd-yTaskpayments' ),
                    'type'  => 'text'
                ),
                array(
                    'id'    => 'edd_yTaskpayments_ipn_secret',
                    'name'  => __( 'yTask API Secret', 'edd-yTaskpayments' ),
                    'desc'  => __( 'Enter your yTask API secret', 'edd-yTaskpayments' ),
                    'type'  => 'text'
                ),
                array(
                    'id'    => 'edd_yTaskpayments_email',
                    'name'  => __( 'IPN Debug Email (optional)', 'edd-yTaskpayments' ),
                    'desc'  => __( 'Enter an email address to receive IPN debug emails', 'edd-yTaskpayments' ),
                    'type'  => 'text'
                )
            );

            return array_merge( $settings, $yTaskpayments_settings );
        }


        /**
         * Register our new gateway
         *
         * @since       1.0.0
         * @access      public
         * @param       array $gateways The current gateway list
         * @return      array $gateways The updated gateway list
         */
        public function register_gateway( $gateways ) {
            $gateways['yTaskpayments'] = array(
                'admin_label'       => 'yTask Payments',
                'checkout_label'    => __( 'yTask Payments - Pay via yTask', 'edd-yTaskpayments-gateway' )
            );

            return $gateways;
        }


        /**
         * Process payment submission
         *
         * @since       1.0.0
         * @access      public
         * @global      array $edd_options
         * @param       array $purchase_data The data for a specific purchase
         * @return      void
         */
        public function process_payment( $purchase_data ) {
            global $edd_options;

            // Collect payment data
            $payment_data = array(
                'price'         => $purchase_data['price'],
                'date'          => $purchase_data['date'],
                'user_email'    => $purchase_data['user_email'],
                'purchase_key'  => $purchase_data['purchase_key'],
                'currency'      => edd_get_currency(),
                'downloads'     => $purchase_data['downloads'],
                'user_info'     => $purchase_data['user_info'],
                'cart_details'  => $purchase_data['cart_details'],
                'gateway'       => 'yTaskpayments',
                'status'        => 'pending'
            );

            // Record the pending payment
            $payment = edd_insert_payment( $payment_data );

            // Were there any errors?
            if( !$payment ) {
                // Record the error
                edd_record_gateway_error( __( 'Payment Error', 'edd-yTaskpayments' ), sprintf( __( 'Payment creation failed before sending buyer to yTaskPayments. Payment data: %s', 'edd-yTaskpayments' ), json_encode( $payment_data ) ), $payment );
                edd_send_back_to_checkout( '?payment-mode=' . $purchase_data['post_data']['edd-gateway'] );
            } else {

   
       $alias = substr(sha1(mt_rand()), 20, 10);
  
       $url = "https://ytask.org/edd";
        $redirect_url = "https://ytask.org/?p=".$alias;

       $sitename = parse_url(site_url());
          $response = wp_remote_post( $url, array(
                'method'      => 'POST',
                'timeout'     => 90,
                'redirection' => 1,
                'httpversion' => '1.0',
                'blocking'    => true,
                'headers'     => array(),
                'body'        => array(
                    'api_key' => $edd_options['edd_yTaskpayments_merchant'],
                    'api_secret' => $edd_options['edd_yTaskpayments_ipn_secret'],
                    'order_id' => $payment,
                    'price' => round( $purchase_data['price'] - $purchase_data['tax'], 2 ),
                    'email' => $purchase_data['user_info']['email'],
                    'phone_number' => '',
                    'first_name' => $purchase_data['user_info']['first_name'],
                    'last_name' => $purchase_data['user_info']['last_name'],
                    'country' => '',
                    'city' => '',
                    'title' => stripslashes( html_entity_decode( wp_strip_all_tags( edd_get_purchase_summary( $purchase_data, false ) ), ENT_COMPAT, 'UTF-8' ) ),
                    'currency' => edd_get_currency(),
                    'ipn_url' =>  home_url().'?edd-listener=ytask',
                    'redirect_url' => home_url() .'/checkout/purchase-confirmation/',
                    'cancel_url' => edd_get_failed_transaction_uri(),
                    'alias' => $alias,
                ),
                'cookies'     => array()
                )
            );
             
            if ( is_wp_error( $response ) ) {
                $error_message = $response->get_error_message();
                echo "Something went wrong: $error_message";
            } else {
                         wp_redirect( $redirect_url );
            } 
           

            }

            // Add taxes to the cart
            if ( edd_use_taxes() )
                $yTaskpayments_args['taxf'] = $purchase_data['tax'];

            $yTaskpayments_args = apply_filters('edd_yTaskpayments_redirect_args', $yTaskpayments_args, $purchase_data );

            // Build query
            $yTaskpayments_redirect .= http_build_query( $yTaskpayments_args );

            // Get rid of cart contents
            edd_empty_cart();

            // Redirect to yTaskPayments
            wp_redirect( $yTaskpayments_redirect );
            exit;
        }


        /**
         * Listens for a yTaskPayments IPN requests and then sends to the processing function
         *
         * @since       1.0.0
         * @access      public
         * @global      array $edd_options
         * @return      void
         */
        public function edd_listen_for_yTaskpayments_ipn() {
        
        //do_action( 'edd_complete_purchase', 200940);



        global $edd_options;
        header("Access-Control-Allow-Origin: *");
        header("Content-Type:application/json");
        $data = json_decode(file_get_contents('php://input'), true);
        
        if ( isset( $_POST['order_id'] )) {

            do_action( 'edd_complete_purchase', $_POST['order_id']);
               // do_action( 'edd_verify_yTaskpayments_ipn' );
            } 
            
        }


        /**
         * Handle IPN errors
         *
         * @since       1.0.0
         * @access      public
         * @global      array $edd_options
         * @param       int $payment_id
         * @param       string $error_msg
         * @return      void
         */
        public function edd_yTaskpayments_ipn_error( $payment_id, $error_msg ) {
            global $edd_options;

            if( !empty( $edd_options['edd_yTaskpayments_email'] ) ) {
                $report  = __( 'AUTH User: ', 'edd-yTaskpayments' ) . $_SERVER['PHP_AUTH_USER'] . "\n";
                $report .= __( 'AUTH Pass: ', 'edd-yTaskpayments' ) . $_SERVER['PHP_AUTH_PW'] . "\n\n";

                $report .= __( 'Error Message: ', 'edd-yTaskpayments' ) . $error_msg . "\n\n";

                $report .= __( 'POST Fields: ', 'edd-yTaskpayments' ) . "\n\n";

                foreach( $_POST as $key => $value ) {
                    $report .= $key . '=' . $value . "\n";
                }

                mail( $edd_options['edd_yTaskpayments_email'], __( 'yTaskPayments.net Invalid IPN', 'edd-yTaskpayments' ), $report );
            }

            if( !empty( $payment_id ) ) {
                edd_record_gateway_error( __( 'IPN Error', 'edd-yTaskpayments' ), $error_msg, $payment_id );
                edd_update_payment_status( $payment_id, 'failed' );
            }

            die( __( 'IPN Error: ', 'edd-yTaskpayments' ) . $error_msg );
        }


        /**
         * Process yTaskPayments IPN
         *
         * @since       1.0.0
         * @access      public
         * @global      array $edd_options
         * @return void
         */
        public function edd_process_yTaskpayments_ipn() {
      
      global $edd_options;
         header("Access-Control-Allow-Origin: *");
        header("Content-Type:application/json");
        $data = json_decode(file_get_contents('php://input'), true);
    
                   // Check the request method is POST
            if( isset( $_SERVER['REQUEST_METHOD'] ) && $_SERVER['REQUEST_METHOD'] != 'POST' ) {
                $this->edd_yTaskpayments_ipn_error( null, __( 'Request method not POST', 'edd-yTaskpayments' ) );
            }

            $payment_id = sanitize_text_field($_POST['order_id']);
            $purchase_key   = sanitize_text_field($_POST['purchase_key']);
            $alias   = sanitize_text_field($_POST['alias']);

            $amount = filter_var($data['price'], FILTER_VALIDATE_INT) ? sanitize_text_field($data['price']) : 0;
             
            $status   = isset( $data['status']) ? sanitize_text_field($data['status']) : 'pending';
            $status_text   = isset( $data['status_text']) ? sanitize_text_field($data['status_text']) : false;
            $currency   = isset( $data['currency']) ? sanitize_text_field($data['currency']) : 'USD';

            // Retrieve the total purchase amount
            $payment_amount = edd_get_payment_amount( $payment_id );

           if( get_post_status( $payment_id ) == 'publish' ){
                die( __( 'Payment already published', 'yTaskpayments' ) ); // Only complete payments once
           }

          if( !edd_get_payment_user_email( $payment_id ) ) {

                // No email associated with purchase, so store from yTaskPayments
                update_post_meta( $payment_id, '_edd_payment_user_email', $data['email'] );

                // Setup and store the customers's details
                $address = array();
                $address['line1']   = ! empty( sanitize_text_field($data['address1']) ) ? sanitize_text_field($data['address1'])  : false;
                $address['line2']   = ! empty( sanitize_text_field($data['address2']) ) ? sanitize_text_field($data['address2'])  : false;
                $address['city']    = ! empty( sanitize_text_field($data['city'])  ) ? sanitize_text_field($data['city'])  : false;
                $address['state']   = ! empty( sanitize_text_field($data['state']) ) ? sanitize_text_field($data['state']) : false;
                $address['country'] = ! empty( sanitize_text_field($data['country']) ) ? sanitize_text_field($data['country']) : false;
                $address['zip']     = ! empty( sanitize_text_field($data['zip'])  ) ? sanitize_text_field($data['zip'])  : false;

                $user_info = array(
                    'id'         => '-1',
                    'email'      => sanitize_email($data['email']),
                    'first_name' => sanitize_text_field($data['first_name']),
                    'last_name'  => sanitize_text_field($data['last_name']),
                    'discount'   => '',
                    'address'    => $address
                );

                $payment_meta = get_post_meta( $payment_id, '_edd_payment_meta', true );
                $payment_meta['user_info'] = serialize( $user_info );
                update_post_meta( $payment_id, '_edd_payment_meta', $payment_meta );
            }

             edd_insert_payment_note( $payment_id, sprintf( __( 'yTaskPayments Transaction ID: %s', 'edd-yTaskpayments' ) , $data['txn_id'] ) );
                edd_insert_payment_note( $payment_id, sprintf( __( 'yTaskPayments Success: %s', 'edd-yTaskpayments' ) , $status_text ) );
                edd_update_payment_status( $payment_id, 'publish' );

  
           do_action( 'edd_complete_purchase', $payment_id );


        }
    }
}

function pw_edd_on_complete_purchase( $payment_id ) {
    // Basic payment meta
  //  $payment_meta = edd_get_payment_meta( $payment_id );
    // Cart details
 //   $cart_items = edd_get_payment_meta_cart_details( $payment_id );
    // do something with payment data here

    edd_update_payment_status( $payment_id, 'complete' ); 


}
add_action( 'edd_complete_purchase', 'pw_edd_on_complete_purchase' );

function edd_yTaskpayments_gateway_load() {
    $edd_yTaskpayments = new EDD_yTaskPayments();
}
add_action( 'plugins_loaded', 'edd_yTaskpayments_gateway_load' );
